@@ -61,7 +61,7 @@ auto.ces <- function(data, models=c("none","simple","full"),
                 intervals=c("none","parametric","semiparametric","nonparametric"), level=0.95,
                 intermittent=c("none","auto","fixed","croston","tsb","sba"), imodel="MNN",
                 bounds=c("admissible","none"),
-                silent=c("none","all","graph","legend","output"),
+                silent=c("all","graph","legend","output","none"),
                 xreg=NULL, xregDo=c("use","select"), initialX=NULL,
                 updateX=FALSE, persistenceX=NULL, transitionX=NULL, ...){
 # Function estimates several CES models in state-space form with sigma = error,
