@@ -232,7 +232,8 @@ auto.ssarima <- function(data, orders=list(ar=c(3,3),i=c(2,1),ma=c(3,3)), lags=c
     lags <- sort(lags,decreasing=FALSE);
 
 # 1 stands for constant, the other one stands for variance
-    nParamMax <- max(ar.max %*% lags + i.max %*% lags,ma.max %*% lags) + sum(ar.max) + sum(ma.max) + constantCheck + 1;
+    nParamMax <- (1 + max(ar.max %*% lags + i.max %*% lags,ma.max %*% lags)
+                  + sum(ar.max) + sum(ma.max) + constantCheck);
 
 # Try to figure out if the number of parameters can be tuned in order to fit something smaller on small samples
 # Don't try to fix anything if the number of seasonalities is greater than 2
@@ -289,7 +290,12 @@ auto.ssarima <- function(data, orders=list(ar=c(3,3),i=c(2,1),ma=c(3,3)), lags=c
     }
 
 # 1 stands for constant/no constant, another one stands for ARIMA(0,0,0)
-    nModels <- prod(i.max + 1) * (1 + sum(ma.max*(1 + sum(ar.max)))) + 1;
+    if(all(ma.max==0)){
+        nModels <- prod(i.max + 1) * (1 + sum(ar.max)) + constantCheck;
+    }
+    else{
+        nModels <- prod(i.max + 1) * (1 + sum(ma.max*(1 + sum(ar.max)))) + constantCheck;
+    }
     testModel <- list(NA);
 # Array with elements x maxorders x horizon x point/lower/upper
     if(combine){
@@ -399,14 +405,14 @@ auto.ssarima <- function(data, orders=list(ar=c(3,3),i=c(2,1),ma=c(3,3)), lags=c
         }
         if(m==1){
             bestIC <- ICValue;
-            dataI <- testModel$residuals;
+            dataMA <- dataI <- testModel$residuals;
             i.best <- i.orders[d,];
             bestICAR <- bestICI <- bestICMA <- bestIC;
         }
         else{
             if(ICValue < bestICI){
                 bestICI <- ICValue;
-                dataI <- testModel$residuals;
+                dataMA <- dataI <- testModel$residuals;
                 if(ICValue < bestIC){
                     i.best <- i.orders[d,];
                     bestIC <- ICValue;
@@ -579,7 +585,7 @@ auto.ssarima <- function(data, orders=list(ar=c(3,3),i=c(2,1),ma=c(3,3)), lags=c
                             if(silent[1]=="d"){
                                 cat("AR: ");cat(ar.test);cat(", ");
                             }
-                            testModel <- ssarima(dataI, orders=list(ar=ar.test,i=0,ma=0), lags=lags,
+                            testModel <- ssarima(dataMA, orders=list(ar=ar.test,i=0,ma=0), lags=lags,
                                                  constant=FALSE, initial=initialType, cfType=cfType,
                                                  h=h, holdout=FALSE,
                                                  intervals=intervals, level=level,
@@ -630,14 +636,14 @@ auto.ssarima <- function(data, orders=list(ar=c(3,3),i=c(2,1),ma=c(3,3)), lags=c
         }
     }
 
-    m <- m + 1;
-    if(silentText==FALSE){
-        cat(paste0(rep("\b",nchar(round(m/nModels,2)*100)+1),collapse=""));
-        cat(paste0(round((m)/nModels,2)*100,"%"));
-    }
-
 #### Test the constant ####
     if(constantCheck){
+        m <- m + 1;
+        if(silentText==FALSE){
+            cat(paste0(rep("\b",nchar(round(m/nModels,2)*100)+1),collapse=""));
+            cat(paste0(round((m)/nModels,2)*100,"%"));
+        }
+
         if(any(c(ar.best,i.best,ma.best)!=0)){
             testModel <- ssarima(data, orders=list(ar=(ar.best),i=(i.best),ma=(ma.best)), lags=(lags),
                                  constant=FALSE, initial=initialType, cfType=cfType,
